package com.ym.mec.vo;

import com.alibaba.fastjson.JSON;
import com.dayaedu.cbs.common.enums.music.EAudioType;
import com.dayaedu.cbs.common.enums.music.EEvaluationStandard;
import com.dayaedu.cbs.common.enums.music.EMusicAvailableType;
import com.dayaedu.cbs.common.enums.music.EMusicPlayMode;
import com.dayaedu.cbs.common.enums.music.EMusicSourceType;
import com.dayaedu.cbs.common.enums.school.EMusicSheetType;
import com.dayaedu.cbs.openfeign.wrapper.music.CbsMusicSheetWrapper;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.biz.dal.wrapper.InstrumentWrapper;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * 曲谱表
 * 2022-12-07 10:16:58
 */
@ApiModel(value = "MusicSheetVo对象", description = "曲谱表查询视图对象")
public class MusicSheetVo {

    @Data
    @ApiModel(" MusicSheetQuery-曲谱表")
    public static class MusicSheetQuery implements QueryInfo {

        @ApiModelProperty("当前页")
        private Integer page;

        @ApiModelProperty("分页行数")
        private Integer rows;

        @ApiModelProperty("关键字匹配")
        private String keyword;

        @ApiModelProperty("仅匹配资源名称")
        private String name;

        @ApiModelProperty("声部Id")
        private String musicSubject;

        @ApiModelProperty("曲目乐器Id")
        private Long musicalInstrumentId;

        @ApiModelProperty("曲谱分类编号")
        private Integer musicSheetCategoriesId;

        private List<Integer> musicSheetCategoriesIdList;

        @ApiModelProperty(value = "删除标识", hidden = true)
        private Boolean delFlag;

        @ApiModelProperty(value = "启用停用标识，曲目状态（0：停用，1：启用）")
        private Boolean status;

        @ApiModelProperty(value = "排序 0:默认 1:首页热门，2热门列表")
        private Integer sortType = 0;

        @ApiModelProperty(value = "是否审核数据")
        private Boolean versionFlag = false;

        @ApiModelProperty(value = "用户ID",hidden = true)
        private Long userId;

        @ApiModelProperty(value = "用户类型",hidden = true)
        private String clientType;

        @ApiModelProperty("是否最近练习")
        private Boolean recentFlag = false;

        @ApiModelProperty("最近练习 当前曲目ID")
        private Long excludeMusicId;

        @ApiModelProperty("app版本")
        private String version;

        @ApiModelProperty("platform")
        private String platform;

        @ApiModelProperty("id集合")
        private List<Long> ids;

        @ApiModelProperty("曲目教程id集合")
        private String musicTutorialIds;

        @ApiModelProperty("曲目标签id集合")
        private String musicTagIds;

        @ApiModelProperty("内容平台ID为null")
        private Boolean cbsIdIsNull;

        @ApiModelProperty("是否转换")
        private Boolean convertFlag;

        @ApiModelProperty("教材版本ID/ 曲目标签ID")
        private Long bookVersionId;

        @ApiModelProperty("音频类型，SING：演唱，PLAY：演奏")
        private List<String> audioPlayTypes;

        @ApiModelProperty("标签分类搜索 HOT 热门，NEW 最新，RECOMMEND 推荐")
        private String searchType;

        @ApiModelProperty("是否只查询基础信息")
        private Boolean basicFlag = false;

        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static MusicSheetQuery from(String json) {
            return JSON.parseObject(json, MusicSheetQuery.class);
        }
    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" MusicSheet-曲谱表")
    public static class MusicSheet {

		@ApiModelProperty("主键")
        private Long id;

        @ApiModelProperty("曲谱分类")
        private Integer musicSheetCategoriesId;

//        @ApiModelProperty(value = "曲谱类型(SINGLE:单曲，CONCERT:合奏)")
//        private EMusicSheetType musicSheetType;

        @ApiModelProperty("曲谱分类名称")
        private String musicSheetCategoriesName;

		@ApiModelProperty("曲谱名称")
        private String musicSheetName;

		@ApiModelProperty("封面图")
        private String titleImg;

		@ApiModelProperty("曲谱来源id")
        private Long userId;

//		@ApiModelProperty("曲目来源类型 TEACHER 老师 PLATFORM 平台")
//        private ESourceFrom sourceType;

        @ApiModelProperty("多声轨选择")
        private String multiTracksSelection;

        @ApiModelProperty("是否公开")
        private Boolean openFlag;

        @ApiModelProperty("作曲人")
        private String composer;

        @ApiModelProperty("上传人")
        private String uploadUser;

		@ApiModelProperty("曲谱声部(适用的乐器,可能是多个,用逗号分隔)")
        private String musicSubject;

		@ApiModelProperty("mp3曲谱类型  MP3;MP3_METRONOME")
        private String mp3Type;

		@ApiModelProperty("曲目图片")
        private String musicImg;

		@ApiModelProperty("曲谱音频文件类型(midi;mp3)")
        private EAudioType audioType;

		@ApiModelProperty("是否可以转简谱（0:否，1：是）")
        private Integer notation;

		@ApiModelProperty("曲谱标签(多个标签用逗号分隔)")
        private String musicTag;

		@ApiModelProperty("默认播放速度")
        private String playSpeed;

		@ApiModelProperty("是否可以评测(0:否;1:是)")
        private Boolean canEvaluate;

        @ApiModelProperty("是否支持混音节拍器")
        private Boolean isMixBeat;

		@ApiModelProperty("伴奏类型  HOMEMODE：自制 COMMON：普通")
        private String accompanimentType;

		@ApiModelProperty("是否展示指法(0:否;1:是)")
        private Boolean showFingering;

		@ApiModelProperty("曲目状态（0：停用，1：启用）")
        private Boolean status;

		@ApiModelProperty("曲目停用原因")
        private String reason;

		@ApiModelProperty("排序号")
        private Integer sortNumber;

		@ApiModelProperty("midi路径")
        private String midiUrl;

		@ApiModelProperty("伴奏含节拍器路径")
        private String metronomeUrl;

		@ApiModelProperty("伴奏（不含节拍器）路径")
        private String url;

		@ApiModelProperty("音频文件存储路径")
        private String audioFileUrl;

		@ApiModelProperty("XML文件存储路径")
        private String xmlFileUrl;

		@ApiModelProperty("是否自带节拍器(0:否;1:是)")
        private Boolean hasBeat;

		@ApiModelProperty("曲目配置")
        private String extConfigJson;


        @ApiModelProperty("是否节奏音乐")
        private Boolean rhythmFlag;


        @ApiModelProperty("创建时间")
        private Date createTime;

		@ApiModelProperty("创建人(老师或者是后台平台用户)")
        private Long createBy;

        @ApiModelProperty("创建用户")
        private String createUser;

		@ApiModelProperty("更新时间")
        private Date updateTime;

		@ApiModelProperty("更新人(老师或者是后台平台用户)")
        private Long updateBy;

        @ApiModelProperty("更新用户")
        private String updateUser;

		@ApiModelProperty("备注")
        private String remark;

		@ApiModelProperty("假删除标识 0：未删除 1：已删除")
        private Boolean delFlag;

        @ApiModelProperty("伴奏")
        private List<MusicSheetAccompanimentVo.MusicSheetAccompaniment> background;

        @ApiModelProperty("曲目缓存")
        private String musicJSON;

        @ApiModelProperty("曲目缓存svg")
        private String musicSvg;

        @ApiModelProperty("曲目缓存简谱svg")
        private String musicJianSvg;

        @ApiModelProperty("曲目缓存首调svg")
        private String musicFirstSvg;


        @ApiModelProperty("被使用次数")
        private Long usedNum = 0L;

        @ApiModelProperty("曲目赏析")
        private String appreciationUrl;

        @ApiModelProperty("曲目创作ID")
        private Long musicSheetCreationId;

        @ApiModelProperty(" 可用乐器(多个,用逗号分隔)")
        private String cbsMusicalInstrumentIds;



        @ApiModelProperty(" 可用乐器(多个,用逗号分隔)")
        private String cbsMusicalInstrumentIdNames;

        @ApiModelProperty("内容平台曲目ID")
        private Long cbsMusicSheetId;

        @ApiModelProperty("首调图片")
        private String musicFirstImg;
        @ApiModelProperty("固定调图片")
        private String musicJianImg;


        @ApiModelProperty("五线谱PDF文件")
        private String musicPdfUrl;


        @ApiModelProperty("首调PDF文件")
        private String firstPdfUrl;


        @ApiModelProperty("固定调PDF文件")
        private String jianPdfUrl;



        @ApiModelProperty("曲目类型(SINGLE:单曲 CONCERT:合奏)")
        private String musicSheetType;

        @ApiModelProperty("乐器信息")
        private List<InstrumentWrapper.Instrument> musicalInstruments;

        @ApiModelProperty("收费类型(FREE:免费;VIP:会员;CHARGE:单曲收费)")
        private String paymentType;



        @ApiModelProperty("是否可转谱(0:否; 1:是)")
        private Boolean isConvertibleScore;

        @ApiModelProperty("谱面类型")
        private String scoreType;


        @ApiModelProperty("音频类型，SING：演唱，PLAY：演奏")
        private String audioPlayTypes;


        @ApiModelProperty("是否总谱渲染")
        private Boolean isScoreRender;

        @ApiModelProperty("演奏是否默认展示总谱渲染")
        private Boolean defaultScoreRender;


        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static MusicSheet from(String json) {
            return JSON.parseObject(json, MusicSheet.class);
        }


        public MusicSheet createBy(String createBy) {
            this.createBy = Long.parseLong(createBy);
            return this;
        }

        public MusicSheet createUser(String createUser) {
            this.createUser = createUser;
            return this;
        }

        public MusicSheet updateBy(String updateBy) {
            this.updateBy = Long.parseLong(updateBy);
            return this;
        }

        public MusicSheet updateUser(String updateUser) {
            this.updateUser = updateUser;
            return this;
        }

        public MusicSheet notation(Integer notation) {
            this.notation = notation;
            return this;
        }

        public MusicSheet userId(String userId) {
            this.userId = Long.valueOf(userId);
            return this;
        }
    }


    @Data
    @EqualsAndHashCode(callSuper = true)
    public static class MusicSheetDetail extends CbsMusicSheetWrapper.MusicSheet {
        @ApiModelProperty
        private Long bizMusicCategoryId;

        @ApiModelProperty("收费类型(FREE:免费;VIP:会员;CHARGE:单曲收费)")
        private String paymentType;

        @ApiModelProperty("是否可转谱(0:否; 1:是)")
        private Boolean isConvertibleScore;

        @ApiModelProperty("谱面类型")
        private String scoreType;


        @ApiModelProperty("使用状态 试用： FREE 锁定： LOCK 解锁：UNLOCK")
        private String useStatus;

        @ApiModelProperty("是否是特殊打击乐")
        private Boolean specialPercussionFlag;

    }

    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" MusicSheet-曲谱表")
    public static class OpenMusicSheetDetail implements Serializable {

        @ApiModelProperty("主键")
        private Long id;

        @ApiModelProperty("业务平台曲目ID")
        private Long bizId;

        @ApiModelProperty("曲目分类ID")
        private Long musicCategoryId;

        @ApiModelProperty("曲目分类名称")
        private String musicCategoryName;

        @ApiModelProperty("曲目标签")
        private String musicTag;

        @ApiModelProperty("曲目标签名称")
        private String musicTagNames;

        @ApiModelProperty("曲目封面图")
        private String musicCover;

        @ApiModelProperty("曲目名称")
        private String name;

        @ApiModelProperty("是否审核版本")
        private Boolean appAuditFlag;

        @ApiModelProperty("曲目声部(多个,用逗号分隔)")
        private String subjectIds;

        @ApiModelProperty("曲目声部code")
        private String subjectCodes;

        @ApiModelProperty("曲目声部名称")
        private String subjectNames;

        @ApiModelProperty("曲目乐器(多个,用逗号分隔)")
        private String musicalInstrumentIds;

        @ApiModelProperty("曲目乐器code")
        private String musicalInstrumentIdCodes;

        @ApiModelProperty("曲目乐器名称")
        private String musicalInstrumentNames;

        @ApiModelProperty("作曲人/音乐人")
        private String composer;

        @ApiModelProperty("曲目类型(SINGLE:单曲 CONCERT:合奏)")
        private EMusicSheetType musicSheetType;

        @ApiModelProperty("是否使用系统节拍器(0:否;1:是)")
        private Boolean isUseSystemBeat;

        @ApiModelProperty("是否可评测(0:否;1:是)")
        private Boolean isEvaluated;

        @ApiModelProperty("是否可转谱(0:否; 1:是)")
        private Boolean isConvertibleScore;

        @ApiModelProperty("是否展示指法(0:否;1:是)")
        private Boolean isShowFingering;

        @ApiModelProperty("是否播入节拍器(0: 否 1:是)")
        private Boolean isPlayBeat;

        @ApiModelProperty("多声轨选择")
        private String multiTracksSelection;

        @ApiModelProperty("播放速度")
        private String playSpeed;

        @ApiModelProperty("播放模式(MIDI;MP3)")
        private EMusicPlayMode playMode;

        @ApiModelProperty("XML文件")
        private String xmlFileUrl;

        @ApiModelProperty("MIDI文件(保留字段)")
        private String midiFileUrl;

        @ApiModelProperty("曲目JSON")
        private String musicJson;

        @ApiModelProperty("曲目缓存JSON(后续使用)")
        private String musicSvgJson;

        @ApiModelProperty("曲目缓存")
        private String musicSvg;

        @ApiModelProperty("评测标准 节奏 AMPLITUDE 音准 FREQUENCY 分贝 DECIBELS")
        private EEvaluationStandard evaluationStandard;

        @ApiModelProperty("简谱缓存")
        private String musicJianSvg;

        @ApiModelProperty("首调缓存")
        private String musicFirstSvg;

        @ApiModelProperty("谱面图片JSON(后续使用)")
        private String musicImgJson;

        @ApiModelProperty("五线谱图片")
        private String musicImg;

        @ApiModelProperty("首调图片")
        private String musicFirstImg;

        @ApiModelProperty("固定调图片")
        private String musicJianImg;

        @ApiModelProperty("谱面参数设置")
        private String extConfigJson;

        @ApiModelProperty("谱面样式参数设置")
        private String extStyleConfigJson;

        @ApiModelProperty("可用类型(PLATFORM: 平台; ORG: 机构; PERSON: 个人)")
        private EMusicAvailableType availableType;

        @ApiModelProperty("来源类型/作者属性(PLATFORM: 平台; ORG: 机构; PERSON: 个人)")
        private EMusicSourceType sourceType;

        @ApiModelProperty("音频类型(HOMEMODE: 自制 COMMON: 普通)")
        private EAudioType audioType;

        @ApiModelProperty("曲目状态（0：停用，1：启用）")
        private Boolean status;

        @ApiModelProperty("停用原因")
        private String reason;

        @ApiModelProperty("备注")
        private String remark;

        @ApiModelProperty("逻辑删除标识(0: 未删 1:已删)")
        private Boolean delFlag;

        @ApiModelProperty("评测频率,详情返回，指定应用端")
        private String evaluationFrequency;

        @ApiModelProperty("乐器信息")
        private List<CbsMusicSheetWrapper.MusicalInstrument> musicalInstruments;

        @ApiModelProperty
        private Long bizMusicCategoryId;

        @ApiModelProperty("收费类型(FREE:免费;VIP:会员;CHARGE:单曲收费)")
        private String paymentType;

        public static OpenMusicSheetDetail from(String json) {
            return JSON.parseObject(json, OpenMusicSheetDetail.class);
        }
    }

    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" MusicSheet-曲谱表")
    public static class OpenMusicSheet implements Serializable {

        @ApiModelProperty("主键")
        private Long id;

        @ApiModelProperty("曲谱分类")
        private Integer musicSheetCategoriesId;

        @ApiModelProperty("曲谱分类名称")
        private String musicSheetCategoriesName;

        @ApiModelProperty("曲谱名称")
        private String musicSheetName;

        @ApiModelProperty("封面图")
        private String titleImg;

//        @ApiModelProperty("曲目来源类型 TEACHER 老师 PLATFORM 平台")
//        private ESourceFrom sourceType;

        @ApiModelProperty("多声轨选择")
        private String multiTracksSelection;

        @ApiModelProperty("是否公开")
        private Boolean openFlag;

        @ApiModelProperty("作曲人")
        private String composer;

        @ApiModelProperty("曲谱声部(适用的乐器,可能是多个,用逗号分隔)")
        private String musicSubject;

        @ApiModelProperty("mp3曲谱类型  MP3;MP3_METRONOME")
        private String mp3Type;

        @ApiModelProperty("曲目图片")
        private String musicImg;

        @ApiModelProperty("曲谱音频文件类型(midi;mp3)")
        private EAudioType audioType;

        @ApiModelProperty("是否可以转简谱（0:否，1：是）")
        private Integer notation;

        @ApiModelProperty("曲谱标签(多个标签用逗号分隔)")
        private String musicTag;

        @ApiModelProperty("默认播放速度")
        private String playSpeed;

        @ApiModelProperty("是否可以评测(0:否;1:是)")
        private Boolean canEvaluate;

        @ApiModelProperty("伴奏类型  HOMEMODE：自制 COMMON：普通")
        private String accompanimentType;

        @ApiModelProperty("是否展示指法(0:否;1:是)")
        private Boolean showFingering;

        @ApiModelProperty("曲目状态（0：停用，1：启用）")
        private Boolean status;

        @ApiModelProperty("曲目停用原因")
        private String reason;

        @ApiModelProperty("排序号")
        private Integer sortNumber;

        @ApiModelProperty("midi路径")
        private String midiUrl;

        @ApiModelProperty("XML文件存储路径")
        private String xmlFileUrl;

        @ApiModelProperty("是否自带节拍器(0:否;1:是)")
        private Boolean hasBeat;

        @ApiModelProperty("曲目配置")
        private String extConfigJson;

        @ApiModelProperty("是否节奏音乐")
        private Boolean rhythmFlag;

        @ApiModelProperty("备注")
        private String remark;

        @ApiModelProperty("假删除标识 0：未删除 1：已删除")
        private Boolean delFlag;

        @ApiModelProperty("曲目缓存")
        private String musicJSON;

        @ApiModelProperty("曲目缓存svg")
        private String musicSvg;

        @ApiModelProperty("曲目缓存简谱svg")
        private String musicJianSvg;

        @ApiModelProperty("曲目缓存首调svg")
        private String musicFirstSvg;

        @ApiModelProperty("被使用次数")
        private Long usedNum = 0L;

        @ApiModelProperty("曲目赏析")
        private String appreciationUrl;

        @ApiModelProperty("曲目创作ID")
        private Long musicSheetCreationId;

        @ApiModelProperty(" 可用乐器(多个,用逗号分隔)")
        private String cbsMusicalInstrumentIds;

        @ApiModelProperty(" 可用乐器(多个,用逗号分隔)")
        private String cbsMusicalInstrumentIdNames;

        @ApiModelProperty("内容平台曲目ID")
        private Long cbsMusicSheetId;

        @ApiModelProperty("首调图片")
        private String musicFirstImg;

        @ApiModelProperty("固定调图片")
        private String musicJianImg;

        @ApiModelProperty("曲目类型(SINGLE:单曲 CONCERT:合奏)")
        private String musicSheetType;

        /*@ApiModelProperty("乐器信息")
        private List<InstrumentWrapper.Instrument> musicalInstruments;*/

        @ApiModelProperty("收费类型(FREE:免费;VIP:会员;CHARGE:单曲收费)")
        private String paymentType;

        public static OpenMusicSheet from(String json) {
            return JSON.parseObject(json, OpenMusicSheet.class);
        }

    }
}
