package com.ym.mec.biz.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ym.mec.biz.dal.dao.StudentMemberTimeDao;
import com.ym.mec.biz.dal.entity.CloudTeacherOrder;
import com.ym.mec.biz.dal.entity.StudentMemberTime;
import com.ym.mec.biz.dal.entity.StudentMemberUpdateLog;
import com.ym.mec.biz.dal.enums.PeriodEnum;
import com.ym.mec.biz.service.StudentMemberTimeService;
import com.ym.mec.util.date.DateUtil;
import org.joda.time.DateTime;
import org.joda.time.DateTimeUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.util.Date;
import java.util.List;

@Service("studentMemberTimeService")
public class StudentMemberTimeServiceImpl extends ServiceImpl<StudentMemberTimeDao, StudentMemberTime> implements StudentMemberTimeService {

    private final static Logger log = LoggerFactory.getLogger(StudentMemberTimeServiceImpl.class);


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveToCloudTeacherOrder(CloudTeacherOrder cloudTeacherOrder) {
        StudentMemberTime one = this.lambdaQuery()
            .eq(StudentMemberTime::getMemberRankId, cloudTeacherOrder.getLevel())
            .eq(StudentMemberTime::getStudentId, cloudTeacherOrder.getStudentId())
            .last("limit 1").one();
        if (one == null) {
            one = new StudentMemberTime();
            one.setMemberRankId(cloudTeacherOrder.getLevel());
            one.setStudentId(cloudTeacherOrder.getStudentId());
            one.setEndTime(cloudTeacherOrder.getEndTime());
            this.save(one);
        }else {
            one.setEndTime(cloudTeacherOrder.getEndTime());
            this.updateById(one);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveToStudentMemberUpdateLog(StudentMemberUpdateLog studentMemberUpdateLog) {
        StudentMemberTime one = this.lambdaQuery()
            .eq(StudentMemberTime::getMemberRankId, studentMemberUpdateLog.getLevel())
            .eq(StudentMemberTime::getStudentId, studentMemberUpdateLog.getStudentId())
            .last("limit 1").one();
        //扣减
        LocalDate nowDate = LocalDate.now().plusDays(1);
        // 转 Date
        Date now = DateUtil.localDateToDate(nowDate);
        if(studentMemberUpdateLog.getBuyType() == 3){
            if (one == null) {
                throw new RuntimeException("学练宝信息不存在");
            }
            studentMemberUpdateLog.setOldEndTime(one.getEndTime());
            if (one.getEndTime().after(new Date())) {
                int i = DateUtil.daysBetweenUp(new Date(), one.getEndTime());
                if (i<studentMemberUpdateLog.getTime()) {
                    throw new RuntimeException("学练宝剩余天数不足");
                }
                studentMemberUpdateLog.setOldEndTime(one.getEndTime());
                one.setEndTime(addDays(one.getEndTime(), -studentMemberUpdateLog.getTime(), studentMemberUpdateLog.getType()));
                this.updateById(one);
            }else {
                throw new RuntimeException("学练宝已过期");
            }
        }else {
            if (one == null) {
                one = new StudentMemberTime();
                one.setMemberRankId(studentMemberUpdateLog.getLevel());
                one.setStudentId(studentMemberUpdateLog.getStudentId());
                one.setEndTime(addDays(now, studentMemberUpdateLog.getTime(), studentMemberUpdateLog.getType()));
                this.save(one);
            }else {
                if (one.getEndTime().after(now)) {
                    studentMemberUpdateLog.setOldEndTime(one.getEndTime());
                    one.setEndTime(addDays(one.getEndTime(), studentMemberUpdateLog.getTime(), studentMemberUpdateLog.getType()));
                }else {
                    studentMemberUpdateLog.setOldEndTime(now);
                    one.setEndTime(addDays(now, studentMemberUpdateLog.getTime(), studentMemberUpdateLog.getType()));
                }
                this.updateById(one);
            }
        }
        studentMemberUpdateLog.setNewEndTime(one.getEndTime());
    }

    @Override
    public List<StudentMemberTime> queryByUserId(List<Integer> userIds) {
        return lambdaQuery()
            .in(StudentMemberTime::getStudentId, userIds)
            .gt(StudentMemberTime::getEndTime, new Date())
            .list();
    }

    private Date addDays(Date endTime, int time, PeriodEnum type) {
        switch (type) {
            case DAY:
                return DateUtil.addDays(endTime, time);
            case MONTH:
                return DateUtil.addMonths(endTime, time);
            case QUARTERLY:
                return DateUtil.addMonths(endTime, time * 3);
            case YEAR_HALF:
                return DateUtil.addMonths(endTime, time * 6);
            case YEAR:
                return DateUtil.addYears(endTime, time);
            default:
                return endTime;
        }
    }
}

